[![pypi](https://img.shields.io/pypi/v/cartolidar.svg)](https://pypi.org/project/cartolidar/)
[![Coverage Status](https://codecov.io/gh/cartolidar/cartolidar/branch/main/graph/badge.svg)](https://codecov.io/gh/cartolidar/cartolidar)
[![Join the chat at https://gitter.im/cartolidar/cartolidar](https://badges.gitter.im/Join%20Chat.svg)](https://gitter.im/cartolidar/cartolidar?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)
[![Binder](https://mybinder.org/badge.svg)](https://mybinder.org/v2/gh/cartolidar/cartolidar/main)

![cartolidar Logo](https://secure.gravatar.com/avatar/ea09c6d439dc57633702164f23b264e5 "clid image")


CartoLidar
----------

Lidar data processing tools focused on Spanish PNOA datasets

> Herramientas para procesado de datos Lidar del PNOA

Lidar PNOA: https://pnoa.ign.es/el-proyecto-pnoa-lidar


Introduction
------------

CartoLidar is a collection of tools to process lidar files "las" and "laz" and
generate other products aimed to forestry and natural environment management.

This project is in alpha version and includes only the "clidtwins" tool.

"clidtwins" searchs for similar areas to a reference one in terms of dasoLidar Variables (DLVs)

DLV: Lidar variables that describe or characterize forest or land cover structure.

> CartoLidar es una coleccin de herramientas destinadas a procesar ficheros lidar
> (las y laz) para clasificar los puntos y generar ficheros rster con DEM y DLVs.
> 
> DEM (Digital Elevation Model): modelos digitales de elevaciones (MDT, MDS)
> 
> DLV (Daso Lidar Variables): variables dasoLidar, que representan diversos
> aspectos de la estructura de una formacin arbolada, arbustiva o de matorral.
> 
> CartoLidar tambin proporciona herramientas adicionales para generar otros
> productos de utilidad en selvicultura y otras areas de gestin del medio
> natural a partir de los ficheros rster con las DLVs. 
> 
> El proyecto est en fase alpha e incluye nicamente la herramienta "clidtwins".
> 
> clidtwins est destinada a buscar zonas similares a una(s) de referencia en
> trminos de determinadas variables dasoLidar (DLVs).


\+ info: [Read the Docs - cartolidar](http://cartolidar-docs.readthedocs.io/en/latest/)
(pgina en construccin)


Install
--------

1. Install official version from [pypi - cartolidar](https://pypi.org/project/cartolidar/):
```
pip install cartolidar
```
or (in case you are working through a proxy server):
```
pip install cartolidar --proxy https://user:password@proxyserver:port
```

2. Download development version from [github - cartolidar](https://github.com/cartolid/cartolidar)

From github you can download the zip version, uncompress it somewhere, i.e.:
```
C:\users\yourUser\Downloads\cartolidar-main\
```
That folder contains a setup.py file (and the other components of the project)
and you can install cartolidar from that directory for your python environment:
```
cd C:\users\yourUser\Downloads\cartolidar-main\
pip install .
```


Requeriments
----
cartolidar requires Python 3.7 or higher.

See other package requirements in requirements.txt.


Use
---

### From the command line (cmd or bash)
a) Run cartolidar package:
```
python -m cartolidar [options]
```
It starts the main menu with the avaliable tools

>Se inicia el menu principal con las herramientas disponibles en cartolidar:
>
>Inicialmente solo est disponible la herramienta qlidtwins


<pre>
    options:
        -h, --help          show this help message and exit</p>
        -h, --help          show this help message and exit
        -v, --verbose       set verbosity level [default: False]
        -V, --version       show program's version number and exit
        -a ACCIONPRINCIPAL  0. Show main menu; 1. qlidtwins: buscar o verificar
                            zonas analogas a una de referencia (con un determinado
                            patron dasoLidar); 2. qlidmerge: integrar ficheros asc
                            de 2x2 km en una capa tif unica (componer mosaico:
                            merge). Default: 0
    other options are tool-specific (see below).
</pre>


b) It is also possible to run a tool (a module) without displayin the main menu (if the module is accesible). E.g.:
```
python qlidtwins.py [options]
```
&nbsp;&nbsp;&nbsp;&nbsp;You can check options with:
```
python qlidtwins.py -h
```
A module is accesible if is called from its folder, is called with path_to_module or its folder is in the PATH environmental variable.


### From python code
You can import packages, modules, classes of functions from a script.py or
within the python interactive interpreter. Examples:
```
import cartolidar
from cartolidar import clidtools
from cartolidar.clidtools import clidtwins
from cartolidar.clidtools.clidtwins import DasoLidarSource
```
To execute module qlidtwins.py from python code:
```
from cartolidar import qlidtwins
```
In this case, there are no options: it runs with qlidtwins.cfg configuration (if exists) or by default.


Required inputs
----
Read [Read the Docs - cartolidar](http://cartolidar-docs.readthedocs.io/en/latest/) for details.

clidtwins reads files in "asc" format (single vector layers) each with a dasoLidar variable (DLV).

> Those files have to be named as: XXX_YYYY_\*IdFileType\*.asc where:
> - XXX, YYYY are UTM coordinates (miles) that identifies the location (the block).
>   - XXX, YYYY are usually the upper-left corner of a 2x2 km square area
> - \*IdFileType\* is any string that includes a DVL identifier (like alt95, fcc05, etc.).
> 
> Example: 318_4738_alt95.asc and 320_4738_alt95.asc are two files with alt95 variable (blocks: 318_4738 and 320_4738).
> 
> If we want to process two blocks (318_4738 and 320_4738) with two DLVs (e.g. alt95 and fcc05), we need these files:
> 318_4738_alt95.asc, 318_4738_fcc05.asc, 320_4738_alt95.asc, 320_4738_fcc05.asc


It's also advisable to include a layer with forest type codes.

> TODO: describe procedure



Use example with python code
----
Procedure:

1. Import package (or Class) and instantiate DasoLidarSource Class:
```
from cartolidar.clidtools.clidtwins import DasoLidarSource
myDasolidar = DasoLidarSource()
```


2. Delimit the prospecting area (optional):
```
myDasolidar.setRangeUTM(
    LCL_marcoCoordMiniX=CFG_marcoCoordMiniX,
    LCL_marcoCoordMaxiX=CFG_marcoCoordMaxiX,
    LCL_marcoCoordMiniY=CFG_marcoCoordMiniY,
    LCL_marcoCoordMaxiY=CFG_marcoCoordMaxiY,
)
```


3. Search for dasoLidar files in the prospecting zone (if any):
> First argument (LCL_listLstDasoVars) is a string with a sequence of DLV identifiers
> and second one (LCL_rutaAscRaizBase) is a path to look for the files with those DLV ids.
```
myDasolidar.searchSourceFiles(
    LCL_listLstDasoVars='Alt95,Fcc05,Fcc03',
    LCL_rutaAscRaizBase='C:/myAscFiles',
)
```
This method creates a property named inFilesListAllTypes. It is a list for every DLV,
each with the list of found file tuples for that file type.
It only includes files of blocks that have all the file types (one file type = one DLV).
Every file tuple consist of a file path and file name.


4. Create a raster (Tiff) file from the DLV found files:

> The createMultiDasoLayerRasterFile method requires the name (with path)
> of the forest type or land cover type vector layer (e.g. Spanish Forest Map -MFE25-)
> and the name of the field (type int) with the forest or land cover type
> identifier (e.g. main species code).
```
myDasolidar.createMultiDasoLayerRasterFile(
    LCL_rutaCompletaMFE='C:/mfe25/24_mfe25.shp',
    LCL_cartoMFEcampoSp='SP1',
)
```

5. Analyze the ranges of every DLV in the reference area:

> The analyzeMultiDasoLayerRasterFile method requires the name (with path)
> of the vector file with the reference polygon for macthing (shp or gpkg).
> If it is ageopackage, the layer name is also required.
```
myDasolidar.analyzeMultiDasoLayerRasterFile(
    LCL_patronVectrName='C:/vector/CorralesPlots.gpkg,
    LCL_patronLayerName='plot01Quercus',
)
```


6. Create new Tiff files with similar zones and proximity to reference one (patron):
```
myDasolidar.generarRasterCluster()
```


to be continued...

<!-- This content will not appear in the rendered Markdown -->


[Ayuda Markdown de github](https://guides.github.com/features/mastering-markdown/)
[Ayuda Markdown de markdownguide](https://www.markdownguide.org/getting-started)

[![Actions Status](https://github.com/cartolidar/cartolidar/workflows/Tests/badge.svg)](https://github.com/cartolidar/cartolidar/actions?query=workflow%3ATests)
